 /*----------------------------------------------------------------------------------------------------+
 |                             National Institute of Standards and Technology                          |
 |                                        Biometric Clients Lab                                        |
 +-----------------------------------------------------------------------------------------------------+
  File author(s):
       Kevin Mangold (kevin.mangold@nist.gov)
 
 +-----------------------------------------------------------------------------------------------------+
 | NOTICE & DISCLAIMER                                                                                 |
 |                                                                                                     |
 | The research software provided on this web site ("software") is provided by NIST as a public        |
 | service. You may use, copy and distribute copies of the software in any medium, provided that you   |
 | keep intact this entire notice. You may improve, modify and create derivative works of the software |
 | or any portion of the software, and you may copy and distribute such modifications or works.        |
 | Modified works should carry a notice stating that you changed the software and should note the date |
 | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards   |
 | and Technology as the source of the software.                                                       |
 |                                                                                                     |
 | The software is expressly provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED,  |
 | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF      |
 | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST        |
 | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR         |
 | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY               |
 | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED |
 | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.                           |
 |                                                                                                     |
 | You are solely responsible for determining the appropriateness of using and distributing the        |
 | software and you assume all risks associated with its use, including but not limited to the risks   |
 | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs   |
 | or equipment, and the unavailability or interruption of operation.  This software is not intended   |
 | to be used in any situation where a failure could cause risk of injury or damage to property.  The  |
 | software was developed by NIST employees.  NIST employee contributions are not subject to copyright |
 | protection within the United States.                                                                |
 |                                                                                                     |
 | Specific hardware and software products identified in this open source project were used in order   |
 | to perform technology transfer and collaboration. In no case does such identification imply         |
 | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
 | does it imply that the products and equipment identified are necessarily the best available for the |
 | purpose.                                                                                            |
 +----------------------------------------------------------------------------------------------------*/

package gov.nist.itl.wsbd.test;

import gov.nist.itl.wsbd.ISensorService;
import gov.nist.itl.wsbd.dom.Dictionary;
import gov.nist.itl.wsbd.dom.Result;
import gov.nist.itl.wsbd.persistence.BindingContext;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.UUID;

import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

public class TestClient implements ISensorService {
	protected String baseUrl;
	
	public TestClient(String baseUrl) {
		this.baseUrl = baseUrl;
	}
	
	public Result call(URL url, String method) throws IOException, JAXBException {
		return call(url, method, null);
	}
	
	public Result call(URL url, String method, Object payload) throws IOException, JAXBException {
		HttpURLConnection connection = (HttpURLConnection)url.openConnection();
		connection.setRequestMethod(method);
		
		if (payload != null) {
			connection.setDoOutput(true);
			
			byte[] data = null;
			
			if (payload instanceof String) {
				data = ((String)payload).getBytes();
			} else if (payload instanceof Dictionary) {
				connection.setRequestProperty("Content-Type", "text/xml");
				
				Marshaller m = BindingContext.getBindingContext(Dictionary.class).createMarshaller();
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				m.marshal(payload, baos);
				data = baos.toByteArray();
			}
			
			//connection.setRequestProperty("Content-Length", String.format("%d", data.length));
			connection.setFixedLengthStreamingMode(data.length);
			connection.getOutputStream().write(data);
		}
		
		Unmarshaller um = BindingContext.getBindingContext(Result.class).createUnmarshaller();
		Result result = (Result)um.unmarshal(connection.getInputStream());

		connection.disconnect();
		return result;
	}
	
	public Result register() {
		return IRegister();
	}
	
	@Override
	public Result IRegister() {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s", baseUrl, "register")), "POST");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}

	public Result unregister(UUID sessionId) {
		return IUnregister(sessionId);
	}
	
	@Override
	public Result IUnregister(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "register", sessionId)), "DELETE");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}

	public Result lock(UUID sessionId) {
		return ILock(sessionId);
	}
	
	@Override
	public Result ILock(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "lock", sessionId)), "POST");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result stealLock(UUID sessionId) {
		return IStealLock(sessionId);
	}

	@Override
	public Result IStealLock(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "lock", sessionId)), "PUT");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result unlock(UUID sessionId) {
		return IUnlock(sessionId);
	}

	@Override
	public Result IUnlock(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "lock", sessionId)), "DELETE");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result getServiceInformation() {
		return IGetServiceInformation();
	}

	@Override
	public Result IGetServiceInformation() {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s", baseUrl, "info")), "GET");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result initialize(UUID sessionId) {
		return IInitialize(sessionId);
	}

	@Override
	public Result IInitialize(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "initialize", sessionId)), "POST", null);
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result getConfiguration(UUID sessionId) {
		return IGetConfiguration(sessionId);
	}

	@Override
	public Result IGetConfiguration(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "configure", sessionId)), "GET");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result setConfiguration(UUID sessionId, Dictionary configuration) {
		return ISetConfiguration(sessionId, configuration);
	}

	@Override
	public Result ISetConfiguration(UUID sessionId, Dictionary configuration) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "configure", sessionId)), "POST", configuration);
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result capture(UUID sessionId) {
		return ICapture(sessionId);
	}

	@Override
	public Result ICapture(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "capture", sessionId)), "POST");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result download(UUID captureId) {
		return IDownload(captureId);
	}

	@Override
	public Result IDownload(UUID captureId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "download", captureId)), "GET");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result getDownloadInformation(UUID captureId) {
		return IGetDownloadInformation(captureId);
	}

	@Override
	public Result IGetDownloadInformation(UUID captureId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s/info", baseUrl, "download", captureId)), "GET");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result thriftyDownload(UUID captureId, String maxSize) {
		return IThriftyDownload(captureId, maxSize);
	}

	@Override
	public Result IThriftyDownload(UUID captureId, String maxSize) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s/%d", baseUrl, "download", captureId, maxSize)), "GET");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	public Result cancel(UUID sessionId) {
		return ICancel(sessionId);
	}

	@Override
	public Result ICancel(UUID sessionId) {
		Result result = null;
		
		try {
			result =  call(new URL(String.format("%s%s/%s", baseUrl, "cancel", sessionId)), "POST");
		} catch (MalformedURLException murle) {
			murle.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (JAXBException jaxbe) {
			jaxbe.printStackTrace();
		}
		
		return result;
	}
	
	@Override
	public int compareTo(ISensorService service) {
		if (this == service) {
			return 0;
		} else {
			return 1;
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		return this == obj;
	}

	@Override
	public InputStream MultipartXMixedReplace() {
		return null;
	}
}