 /*----------------------------------------------------------------------------------------------------+
 |                             National Institute of Standards and Technology                          |
 |                                        Biometric Clients Lab                                        |
 +-----------------------------------------------------------------------------------------------------+
  File author(s):
       Kevin Mangold (kevin.mangold@nist.gov)
 
 +-----------------------------------------------------------------------------------------------------+
 | NOTICE & DISCLAIMER                                                                                 |
 |                                                                                                     |
 | The research software provided on this web site ("software") is provided by NIST as a public        |
 | service. You may use, copy and distribute copies of the software in any medium, provided that you   |
 | keep intact this entire notice. You may improve, modify and create derivative works of the software |
 | or any portion of the software, and you may copy and distribute such modifications or works.        |
 | Modified works should carry a notice stating that you changed the software and should note the date |
 | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards   |
 | and Technology as the source of the software.                                                       |
 |                                                                                                     |
 | The software is expressly provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED,  |
 | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF      |
 | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST        |
 | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR         |
 | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY               |
 | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED |
 | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.                           |
 |                                                                                                     |
 | You are solely responsible for determining the appropriateness of using and distributing the        |
 | software and you assume all risks associated with its use, including but not limited to the risks   |
 | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs   |
 | or equipment, and the unavailability or interruption of operation.  This software is not intended   |
 | to be used in any situation where a failure could cause risk of injury or damage to property.  The  |
 | software was developed by NIST employees.  NIST employee contributions are not subject to copyright |
 | protection within the United States.                                                                |
 |                                                                                                     |
 | Specific hardware and software products identified in this open source project were used in order   |
 | to perform technology transfer and collaboration. In no case does such identification imply         |
 | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
 | does it imply that the products and equipment identified are necessarily the best available for the |
 | purpose.                                                                                            |
 +----------------------------------------------------------------------------------------------------*/

package gov.nist.itl.wsbd.test.tests;

import static gov.nist.itl.wsbd.Logging.trace;
import gov.nist.itl.wsbd.Utility;
import gov.nist.itl.wsbd.dom.Result;
import gov.nist.itl.wsbd.dom.Status;
import gov.nist.itl.wsbd.test.harness.OperationTestHarness;

import java.util.UUID;

import junit.framework.Assert;

import org.junit.Test;

public class LockTests extends OperationTestHarness {

	@Test
	public void LockAndUnlockIsSuccessful() {
		trace("LockTests.LockAndUnlockIsSuccessful()");
		
		UUID id = UUID.randomUUID();
		Assert.assertTrue(Utility.isValidId(id));
		
		Result result = client.lock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
		
		result = client.unlock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
	}
	
	@Test
	public void MultipleLocksUsingSameIdSuccessful() {
		trace("LockTests.MultipleLocksUsingSameIdSuccessful()");
		
		UUID id = UUID.randomUUID();
		Assert.assertTrue(Utility.isValidId(id));
		
		Result result = client.lock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
		
		result = client.lock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
		
		result = client.unlock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
	}
	
	@Test
	public void LockingWhenAlreadyLockedYieldsLockHeldByAnother() {
		trace("LockTests.LockingWhenAlreadyLockedFails()");
		
		UUID id = UUID.randomUUID();
		Assert.assertTrue(Utility.isValidId(id));
		
		Result result = client.lock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());

		result = client.lock(UUID.randomUUID());
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.LOCK_HELD_BY_ANOTHER, result.getStatus());
		
		result = client.unlock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
	}
	
	@Test
	public void StealLockSucceeded() {
		trace("LockTests.StealLockSucceeded()");
		
		UUID id = UUID.randomUUID();
		Assert.assertTrue(Utility.isValidId(id));
		
		Result result = client.lock(id);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
		
		UUID id2 = UUID.randomUUID();
		Assert.assertTrue(Utility.isValidId(id2));
		
		result = client.lock(id2);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.LOCK_HELD_BY_ANOTHER, result.getStatus());
		
		result = client.stealLock(id2);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
		
		result = client.unlock(id2);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(Status.SUCCESS, result.getStatus());
	}
}