 /*----------------------------------------------------------------------------------------------------+
 |                             National Institute of Standards and Technology                          |
 |                                        Biometric Clients Lab                                        |
 +-----------------------------------------------------------------------------------------------------+
  File author(s):
       Kevin Mangold (kevin.mangold@nist.gov)
 
 +-----------------------------------------------------------------------------------------------------+
 | NOTICE & DISCLAIMER                                                                                 |
 |                                                                                                     |
 | The research software provided on this web site ("software") is provided by NIST as a public        |
 | service. You may use, copy and distribute copies of the software in any medium, provided that you   |
 | keep intact this entire notice. You may improve, modify and create derivative works of the software |
 | or any portion of the software, and you may copy and distribute such modifications or works.        |
 | Modified works should carry a notice stating that you changed the software and should note the date |
 | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards   |
 | and Technology as the source of the software.                                                       |
 |                                                                                                     |
 | The software is expressly provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED,  |
 | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF      |
 | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST        |
 | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR         |
 | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY               |
 | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED |
 | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.                           |
 |                                                                                                     |
 | You are solely responsible for determining the appropriateness of using and distributing the        |
 | software and you assume all risks associated with its use, including but not limited to the risks   |
 | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs   |
 | or equipment, and the unavailability or interruption of operation.  This software is not intended   |
 | to be used in any situation where a failure could cause risk of injury or damage to property.  The  |
 | software was developed by NIST employees.  NIST employee contributions are not subject to copyright |
 | protection within the United States.                                                                |
 |                                                                                                     |
 | Specific hardware and software products identified in this open source project were used in order   |
 | to perform technology transfer and collaboration. In no case does such identification imply         |
 | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
 | does it imply that the products and equipment identified are necessarily the best available for the |
 | purpose.                                                                                            |
 +----------------------------------------------------------------------------------------------------*/

package gov.nist.itl.wsbd;

import gov.nist.itl.wsbd.dom.Dictionary;

import java.math.BigInteger;

public class Configuration extends Dictionary {
	private static Configuration currentConfiguration;
	
	public static Configuration getInstance() {
		if (currentConfiguration == null) {
			currentConfiguration = new Configuration();
		}
		
		return currentConfiguration;
	}
	
	//xs:boolean
	//boolean
	public boolean isAutoDropLRUSessions() {
		return (Boolean)get(Constants.autoDropLRUSessionsKey);
	}
	
	public void setAutoDropLRUSessions(boolean value) {
		put(Constants.autoDropLRUSessionsKey, value);
	}
	
	
	//xs:positiveInteger
	//BigInteger
	public BigInteger getCaptureTimeout() {
		return (BigInteger)get(Constants.captureTimeoutKey);
	}
	
	public void setCaptureTimeout(BigInteger value) {
		put(Constants.captureTimeoutKey, value);
	}
	
	
	//xs:positiveInteger
	//BigInteger
	public BigInteger getGetConfigurationTimeout() {
		return (BigInteger)get(Constants.getConfigurationTimeoutKey);
	}
	
	public void setGetConfigurationTimeout(BigInteger value) {
		put(Constants.getConfigurationTimeoutKey, value);
	}
	
	
	//xs:nonNegativeInteger
	//BigInteger
	public BigInteger getInactivityTimeout() {
		return (BigInteger)get(Constants.inactivityTimeoutKey);
	}
	
	public void setInactivityTimeout(BigInteger value) {
		put(Constants.inactivityTimeoutKey, value);
	}
	
	
	//xs:positiveInteger
	//BigInteger
	public BigInteger getInitializationTimeout() {
		return (BigInteger)get(Constants.initializationTimeoutKey);
	}
	
	public void setInitializationTimeout(BigInteger value) {
		put(Constants.initializationTimeoutKey, value);
	}
	
	
	//xs:dateTime
	//javax.xml.datatype.XMLGregorianCalendar
	public void getLastUpdated() {
	}
	
	public void setLastUpdated() {
		//TODO Configuration.setLastUpdated()
	}
	
	
	//xs:nonNegativeInteger
	//BigInteger
	public BigInteger getLockStealingPreventionPeriod() {
		return (BigInteger)get(Constants.lockStealingPreventionPeriodKey);
	}
	
	public void setLockStealingPreventionPeriod(BigInteger value) {
		put(Constants.lockStealingPreventionPeriodKey, value);
	}
	
	
	//xs:boolean
	//boolean
	public boolean isLRUCaptureDataAutomaticallyDropped() {
		return (Boolean)get(Constants.lruCaptureDataAutomaticallyDroppedKey);
	}
	
	public void setLRUCaptureDataAutomaticallyDropped(boolean value) {
		put(Constants.lruCaptureDataAutomaticallyDroppedKey, value);
	}
	
	
	//xs:positiveInteger
	//BigInteger
	public BigInteger getMaximumConcurrentSessions() {
		return (BigInteger)get(Constants.maximumConcurrentSessionsKey);
	}
	
	public void setMaximumConcurrentSessions(BigInteger value) {
		put(Constants.maximumConcurrentSessionsKey, value);
	}
	
	
	//xs:positiveInteger
	//BigInteger
	public BigInteger getMaximumStorageCapacity() {
		return (BigInteger)get(Constants.maximumStorageCapacityKey);
	}
	
	public void setMaximumStorageCapacity(BigInteger value) {
		put(Constants.maximumStorageCapacityKey, value);
	}
	
	
	//xs:string
	//String
	public String getModality() {
		return (String)get(Constants.modalityKey);
	}
	
	public void setModality(String value) {
		put(Constants.modalityKey, value);
	}
	
	
	//xs:string
	//String
	public String getSubmodality() {
		return (String)get(Constants.submodalityKey);
	}
	
	public void setSubmodality(String value) {
		put(Constants.submodalityKey, value);
	}
	
	
	//xs:nonNegativeInteger
	//BigInteger
	public BigInteger getPostAcquisitionProcessingTime() {
		return (BigInteger)get(Constants.postAcquisitionProcessingTimeKey);
	}
	
	public void setPostAcquisitionProcessingTime(BigInteger value) {
		put(Constants.postAcquisitionProcessingTimeKey, value);
	}
	
	
	//xs:positiveInteger
	//BigInteger
	public BigInteger getSetConfigurationTimeout() {
		return (BigInteger)get(Constants.setConfigurationTimeoutKey);
	}
	
	public void setSetConfigurationTimeout(BigInteger value) {
		put(Constants.setConfigurationTimeoutKey, value);
	}
}