 /*----------------------------------------------------------------------------------------------------+
 |                             National Institute of Standards and Technology                          |
 |                                        Biometric Clients Lab                                        |
 +-----------------------------------------------------------------------------------------------------+
  File author(s):
       Kevin Mangold (kevin.mangold@nist.gov)
 
 +-----------------------------------------------------------------------------------------------------+
 | NOTICE & DISCLAIMER                                                                                 |
 |                                                                                                     |
 | The research software provided on this web site ("software") is provided by NIST as a public        |
 | service. You may use, copy and distribute copies of the software in any medium, provided that you   |
 | keep intact this entire notice. You may improve, modify and create derivative works of the software |
 | or any portion of the software, and you may copy and distribute such modifications or works.        |
 | Modified works should carry a notice stating that you changed the software and should note the date |
 | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards   |
 | and Technology as the source of the software.                                                       |
 |                                                                                                     |
 | The software is expressly provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED,  |
 | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF      |
 | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST        |
 | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR         |
 | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY               |
 | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED |
 | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.                           |
 |                                                                                                     |
 | You are solely responsible for determining the appropriateness of using and distributing the        |
 | software and you assume all risks associated with its use, including but not limited to the risks   |
 | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs   |
 | or equipment, and the unavailability or interruption of operation.  This software is not intended   |
 | to be used in any situation where a failure could cause risk of injury or damage to property.  The  |
 | software was developed by NIST employees.  NIST employee contributions are not subject to copyright |
 | protection within the United States.                                                                |
 |                                                                                                     |
 | Specific hardware and software products identified in this open source project were used in order   |
 | to perform technology transfer and collaboration. In no case does such identification imply         |
 | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
 | does it imply that the products and equipment identified are necessarily the best available for the |
 | purpose.                                                                                            |
 +----------------------------------------------------------------------------------------------------*/

package gov.nist.itl.wsbd;

import gov.nist.itl.wsbd.dom.Dictionary;
import gov.nist.itl.wsbd.dom.Result;
import gov.nist.itl.wsbd.streaming.IMixedReplaceable;

import java.util.UUID;

import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;

public interface ISensorService extends IMixedReplaceable, Comparable<ISensorService> {
	
	@POST
	@Produces("text/xml")
	@Path("register")
	Result IRegister();
	
	@DELETE
	@Produces("text/xml")
	@Path("register/{sessionId}")
	Result IUnregister(@PathParam("sessionId") UUID sessionId);
	
	@POST
	@Produces("text/xml")
	@Path("lock/{sessionId}")
	Result ILock(@PathParam("sessionId") UUID sessionId);
	
	@PUT
	@Produces("text/xml")
	@Path("lock/{sessionId}")
	Result IStealLock(@PathParam("sessionId") UUID sessionId);
	
	@DELETE
	@Produces("text/xml")
	@Path("lock/{sessionId}")
	Result IUnlock(@PathParam("sessionId") UUID sessionId);
	
	@GET
	@Produces("text/xml")
	@Path("info")
	Result IGetServiceInformation();
	
	@POST
	@Produces("text/xml")
	@Path("initialize/{sessionId}")
	Result IInitialize(@PathParam("sessionId") UUID sessionId) throws InterruptedException;
	
	@GET
	@Produces("text/xml")
	@Path("configure/{sessionId}")
	Result IGetConfiguration(@PathParam("sessionId") UUID sessionId) throws InterruptedException;
	
	@POST
	@Produces("text/xml")
	@Path("configure/{sessionId}")
	Result ISetConfiguration(@PathParam("sessionId") UUID sessionId, Dictionary configuration) throws InterruptedException;

	@POST
	@Produces("text/xml")
	@Path("capture/{sessionId}")
	Result ICapture(@PathParam("sessionId") UUID sessionId) throws InterruptedException;
	
	@GET
	@Produces("text/xml")
	@Path("download/{captureId}")
	Result IDownload(@PathParam("captureId") UUID captureId) throws InterruptedException;
	
	@GET
	@Produces("text/xml")
	@Path("download/{captureId}/info")
	Result IGetDownloadInformation(@PathParam("captureId") UUID captureId) throws InterruptedException;
	
	@GET
	@Produces("text/xml")
	@Path("download/{captureId}/{maxSize}")
	Result IThriftyDownload(@PathParam("captureId") UUID captureId, @PathParam("maxSize") String maxSize) throws InterruptedException;
	
	@POST
	@Produces("text/xml")
	@Path("cancel/{sessionId}")
	Result ICancel(@PathParam("sessionId") UUID sessionId);
}