 /*----------------------------------------------------------------------------------------------------+
 |                             National Institute of Standards and Technology                          |
 |                                        Biometric Clients Lab                                        |
 +-----------------------------------------------------------------------------------------------------+
  File author(s):
       Kevin Mangold (kevin.mangold@nist.gov)
 
 +-----------------------------------------------------------------------------------------------------+
 | NOTICE & DISCLAIMER                                                                                 |
 |                                                                                                     |
 | The research software provided on this web site ("software") is provided by NIST as a public        |
 | service. You may use, copy and distribute copies of the software in any medium, provided that you   |
 | keep intact this entire notice. You may improve, modify and create derivative works of the software |
 | or any portion of the software, and you may copy and distribute such modifications or works.        |
 | Modified works should carry a notice stating that you changed the software and should note the date |
 | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards   |
 | and Technology as the source of the software.                                                       |
 |                                                                                                     |
 | The software is expressly provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED,  |
 | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF      |
 | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST        |
 | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR         |
 | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY               |
 | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED |
 | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.                           |
 |                                                                                                     |
 | You are solely responsible for determining the appropriateness of using and distributing the        |
 | software and you assume all risks associated with its use, including but not limited to the risks   |
 | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs   |
 | or equipment, and the unavailability or interruption of operation.  This software is not intended   |
 | to be used in any situation where a failure could cause risk of injury or damage to property.  The  |
 | software was developed by NIST employees.  NIST employee contributions are not subject to copyright |
 | protection within the United States.                                                                |
 |                                                                                                     |
 | Specific hardware and software products identified in this open source project were used in order   |
 | to perform technology transfer and collaboration. In no case does such identification imply         |
 | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
 | does it imply that the products and equipment identified are necessarily the best available for the |
 | purpose.                                                                                            |
 +----------------------------------------------------------------------------------------------------*/

package gov.nist.itl.wsbd.concurrency;

import static gov.nist.itl.wsbd.Logging.trace;

import gov.nist.itl.wsbd.Configuration;
import gov.nist.itl.wsbd.Constants;
import gov.nist.itl.wsbd.dom.Status;

import java.util.List;
import java.util.Vector;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

public class OperationManager extends Thread {
	private static OperationManager manager;
	private boolean runCondition = true;
	
	private ExecutorService sensorService;
	private Vector<Operation> sensorOperations;
	private Operation currentOperation = null;
	private Future<?> currentFuture = null;

	private ExecutorService nonSensorService;
	private Vector<Operation> nonSensorOperations;
	
	private static int maxThreadCount = 32;
	
	private OperationManager() {
		trace("OperationManager()");
		
		if (Configuration.getInstance().containsKey(Constants.maximumConcurrentSessionsKey)) {
			//TODO change spec for int vs BigInteger
			
			if (Configuration.getInstance().getMaximumConcurrentSessions().intValue() > 0) {
				maxThreadCount = Configuration.getInstance().getMaximumConcurrentSessions().intValue();
			}
			
			/*if (Configuration.getInstance().getMaximumConcurrentSessions() > 0) {
				maxThreadCount = Configuration.getInstance().getMaximumConcurrentSessions();
			}*/
		}
	
		sensorService = Executors.newSingleThreadExecutor();
		nonSensorService = Executors.newFixedThreadPool(maxThreadCount);
		
		sensorOperations = new Vector<Operation>();
		nonSensorOperations = new Vector<Operation>();
		start();
	}

	public void addOperation(Operation operation, boolean sensorOperation) {
		trace("OperationManager.addOperation(Operation, boolean)");
		
		if (sensorOperation) {
			synchronized (sensorOperations) {
				if (currentFuture == null || currentFuture.isDone()) {  
					sensorOperations.add(operation);
				} else {
					operation.forceResult(Status.SENSOR_BUSY);
				}
			}
		} else {
			synchronized (nonSensorOperations) {
				nonSensorOperations.add(operation);
			}
		}
	}

	public boolean cancelCurrent() {
		trace("OperationManager.cancelCurrent()");
		
		try {
			if (currentFuture != null) {
				//System.err.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return currentFuture.cancel(true);
			} else {
				System.err.println("0564214038706432406874034357354");
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return false;
	}
	
	@Override
	public void run() {
		trace("OperationManager.run()");
		
		while (isRunning()) {
			synchronized (sensorOperations) {
				if (currentFuture == null || currentFuture.isDone()) {
					currentOperation = null;
					currentFuture = null;
					
					if (sensorOperations.size() > 0) {
						Operation op = sensorOperations.remove(0);
						currentOperation = op;
						Future<?> f = sensorService.submit(op);
						currentFuture = f;
					} else {
					}	
				} else {
					
				}
			}
			
			synchronized (nonSensorOperations) {
				while (nonSensorOperations.size() > 0) {
					Operation op = nonSensorOperations.remove(0);
					Future<?> f = nonSensorService.submit(op);
				}
			}
		}
		
		List<Runnable> remainingSensorOperations = sensorService.shutdownNow();
		List<Runnable> remainingNonSensorOperations = nonSensorService.shutdownNow();
	}
	
	public boolean isRunning() {
		return runCondition;
	}
	
	public static OperationManager getManager() {
		trace("OperationManager.getManager()");
		
		if (manager == null)
			manager = new OperationManager();
		
		return manager;
	}
}